#!/usr/bin/perl 

package remote_infosys_shared;

# import resource gateway configuration
use gw_shared qw(%gw_config);

sub mds_valid ($){
   # Mds-validfrom/to
   my ($ttl) = (@_);  
   my $seconds=time;
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($seconds);
   print "Mds-validfrom: ";
   printf "%4d%02d%02d%02d%02d%02d%1s", $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
   ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($seconds+$ttl);
   print "Mds-validto: ";
   printf "%4d%02d%02d%02d%02d%02d%1s", $year+1900,$mon+1,$mday,$hour,$min,$sec,"Z\n";
   print "\n";
}

# TODO: change to remote_read_pbsnodes and reduce this package to only contain 
# remote subroutines. Remaining subroutines should just be used from infosys_shared.
sub read_pbsnodes ($$){ 
   #processing the pbsnodes output by using a hash of hashes %hoh_pbsnodes (referrenced by $hashref)
   my ($pbsnodes_command,$hashref) =@_;

   if($gw_config{gateway_enabled}) {
	write_log("Acting as gateway");
   }

   if(! $gw_config{gateway_enabled}) {
	#print("local pbsnodes query: $pbsnodes_command -a\n");
	unless (open PBSNODESOUT,  "$pbsnodes_command -a |") {
      		write_log("error in executing local pbsnodes");
		die "Error in executing local pbsnodes";
   	}
   } else {
	#print("remote pbsnodes query: $gw_config{remote_exec} $gw_config{gateway_conf} '$pbsnodes_command -a'\n");
	unless (open PBSNODESOUT, "$gw_config{remote_exec} $gw_config{gateway_conf} '$pbsnodes_command -a' |") {
		write_log("error in executing pbsnodes remotely");
      		die "Error in executing pbsnodes remotely";
   	}
   }
   
   while (my $line= <PBSNODESOUT>) {	    
   	 if ($line =~ /^$/) {next}; 
   	 if ($line =~ /^([\w\-]+)/) {		 
   	    $nodeid=$1;
   	    next;	    
   	 }
   	 if ($line =~ / = /)  {
   	     ($node_var,$node_value) = split (/ = /, $line);
   	     $node_var =~ s/\s+//g;
   	     chop $node_value;  	     
   	 }	     
   	 ${$hashref}{$nodeid}{$node_var}=$node_value;     
   } 
   close PBSNODESOUT;
}

sub read_sgenodes ($$){ 
   #processing the SGE qstat -f output by using a hash %h_sgenodes (referrenced by $hashref)
#GD SGE output
## queuename            qtype used/tot. load_avg arch      states
##----------------------------------------------------------------------------
##lxsrv009             B     2/2       2.40     glinux    d

   my ($sgenodes_command,$hashref) =@_;
   unless (open SGENODESOUT,  "$sgenodes_command |") {
      write_log("error in executing $sgenodes_command");
      die "Error in executing $sgenodes_command";
   }
   while (my $line= <SGENODESOUT>) {	    
   	 if ($line =~ /^$/) {next}; 
	 
   	 if ($line =~ /^lxsrv/) {
	     ($nodeid, $nodetype, $rest ) = split(/\s+/, $line);
	     if ( ! ($nodetype =~ /B/ )) {next}; # only nodes for batch

	     ${$hashref}{$nodeid} = $rest;     
	 } 
     }
   close SGENODESOUT;
}


sub read_jobinf ($$){ 
# read detailed SGE jobinfo to obtain resource usage
#
# a call as: qstat -F -j 154138
# gives : usage    1:                  cpu=06:01:07, mem=202.10015 GBs, io=0.00000, vmem=12.94M, maxvmem=12.94M

   my ($jid, $usgref) = @_;
#   print "$sgejobinf_command $jid ";
   $sgejobinf_command = "qstat -F -j "; 
   unless (open SGEJOBINFOUT,  "$sgejobinf_command $jid |") {
      write_log("error in executing $sgejobinf_command");
      die "Error in executing $sgejobinf_command";
   }
   while (my $linein= <SGEJOBINFOUT>) {	    
   	 if ($linein =~ /^$/) {next}; 
	 
   	 if ($linein =~ /usage/) {
	     chomp($linein);
	     ($dum, $usg ) = split(":", $linein,2);
	     $usg =~ s/^\s*//;
	     
	     last;
#	     print "$ usg \n";
	 } 
     }
   ${$usgref} = $usg;
   close SGEJOBINFOUT;
}

sub write_log ($){
  use POSIX;
  #records logging info into the /var/log/infoprovider.log
  my ($message) = @_; 
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime (time);
  my $timestamp=POSIX::strftime("%b %d %T", $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst); 
  my $scriptname= $0; 
  open PROVIDERLOG, '>>/var/log/infoprovider.log';
  print PROVIDERLOG  "$timestamp [$$] $scriptname: $message\n";
  close PROVIDERLOG; 
}  
   
1;
